#!/bin/bash
set -euo pipefail

# Publish Docker image to GitHub Container Registry
# Usage: ./publish-docker.sh [tag] [username] [--platform=platform] [--build-and-push]

GITHUB_USERNAME="anosatsuk124"
TAG="latest"
PLATFORM=""
BUILD_AND_PUSH=false
BUILD_ARGS=()

# Parse arguments
while [[ $# -gt 0 ]]; do
	case $1 in
		--platform=*)
			PLATFORM="${1#*=}"
			shift
			;;
		--platform)
			PLATFORM="$2"
			shift 2
			;;
		--build-and-push)
			BUILD_AND_PUSH=true
			shift
			;;
		--username=*)
			GITHUB_USERNAME="${1#*=}"
			shift
			;;
		-*)
			BUILD_ARGS+=("$1")
			shift
			;;
		*)
			if [[ -z "${TAG_SET:-}" ]]; then
				TAG="$1"
				TAG_SET=true
			elif [[ -z "${USERNAME_SET:-}" ]]; then
				GITHUB_USERNAME="$1"
				USERNAME_SET=true
			else
				BUILD_ARGS+=("$1")
			fi
			shift
			;;
	esac
done

LOCAL_IMAGE="voice-rss-summary:${TAG}"
GHCR_IMAGE_1="ghcr.io/${GITHUB_USERNAME}/voice-rss-summary:${TAG}"
GHCR_IMAGE_2="ghcr.io/${GITHUB_USERNAME}/voicersssummary:${TAG}"

echo "Publishing Docker image to GitHub Container Registry"
echo "Local image: ${LOCAL_IMAGE}"
echo "GHCR images: ${GHCR_IMAGE_1}, ${GHCR_IMAGE_2}"
if [[ -n "$PLATFORM" ]]; then
	echo "Target platform: ${PLATFORM}"
fi

# Build image if requested
if [[ "$BUILD_AND_PUSH" == "true" ]]; then
	echo "Building image first..."
	BUILD_CMD=(./build-docker-image.sh "$TAG")
	if [[ -n "$PLATFORM" ]]; then
		BUILD_CMD+=(--platform "$PLATFORM")
	fi
	if [[ ${#BUILD_ARGS[@]} -gt 0 ]]; then
		BUILD_CMD+=("${BUILD_ARGS[@]}")
	fi
	
	echo "Running: ${BUILD_CMD[*]}"
	if ! "${BUILD_CMD[@]}"; then
		echo "Error: Failed to build image"
		exit 1
	fi
fi

# Check if local image exists
if ! docker image inspect "${LOCAL_IMAGE}" >/dev/null 2>&1; then
	echo "Error: Local Docker image '${LOCAL_IMAGE}' not found"
	if [[ -n "$PLATFORM" ]]; then
		echo "Build it first with: ./build-docker-image.sh ${TAG} --platform=${PLATFORM}"
	else
		echo "Build it first with: ./build-docker-image.sh ${TAG}"
	fi
	echo "Or use --build-and-push flag to build and push in one command"
	exit 1
fi

# Check if user is logged in to GHCR
if ! docker system info | grep -q "ghcr.io"; then
	echo "Checking GHCR authentication..."
	if ! echo "test" | docker login ghcr.io --username "${GITHUB_USERNAME}" --password-stdin >/dev/null 2>&1; then
		echo "Please authenticate with GitHub Container Registry first:"
		echo "1. Create a personal access token with 'write:packages' scope at:"
		echo "   https://github.com/settings/tokens"
		echo "2. Login with: echo \$GITHUB_TOKEN | docker login ghcr.io -u ${GITHUB_USERNAME} --password-stdin"
		echo "   or: docker login ghcr.io -u ${GITHUB_USERNAME}"
		exit 1
	fi
fi

# Tag images for GHCR
echo "Tagging images for GHCR..."
docker tag "${LOCAL_IMAGE}" "${GHCR_IMAGE_1}"
docker tag "${LOCAL_IMAGE}" "${GHCR_IMAGE_2}"

# Push to GHCR
echo "Pushing ${GHCR_IMAGE_1}..."
if docker push "${GHCR_IMAGE_1}"; then
	echo "✅ Successfully pushed ${GHCR_IMAGE_1}"
else
	echo "❌ Failed to push ${GHCR_IMAGE_1}"
	exit 1
fi

echo "Pushing ${GHCR_IMAGE_2}..."
if docker push "${GHCR_IMAGE_2}"; then
	echo "✅ Successfully pushed ${GHCR_IMAGE_2}"
else
	echo "❌ Failed to push ${GHCR_IMAGE_2}"
	exit 1
fi

echo ""
echo "🎉 Successfully published Docker images to GitHub Container Registry!"
echo ""
echo "Images available at:"
echo "  - ${GHCR_IMAGE_1}"
echo "  - ${GHCR_IMAGE_2}"
if [[ -n "$PLATFORM" ]]; then
	echo "Platform: ${PLATFORM}"
fi
echo ""
echo "To run from GHCR:"
if [[ -n "$PLATFORM" && "$PLATFORM" != "$(uname -m)" ]]; then
	echo "  docker run --platform ${PLATFORM} -p 3000:3000 -p 3001:3001 ${GHCR_IMAGE_1}"
else
	echo "  docker run -p 3000:3000 -p 3001:3001 ${GHCR_IMAGE_1}"
fi
echo ""
echo "To use with run-docker.sh:"
echo "  ./run-docker.sh container-name ${TAG} --from-ghcr"